package cn.iocoder.yudao.module.bpm.controller.admin.oa.dimission;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.dimission.vo.OaDimissionPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.dimission.vo.OaDimissionRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.dimission.vo.OaDimissionSaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.dimission.OaDimissionDO;
import cn.iocoder.yudao.module.bpm.service.oa.dimission.OaDimissionService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

@Tag(name = "管理后台 - 离职流程信息")
@RestController
@RequestMapping("/bpm/oa-dimission")
@Validated
public class OaDimissionController {

    @Resource
    private OaDimissionService oaDimissionService;


    @PostMapping("/start")
    @Operation(summary = "提交或暂存离职流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-Dimission:start')")
    public CommonResult<Long> startOaDimission(@Valid @RequestBody OaDimissionSaveReqVO createReqVO) {
        return success(oaDimissionService.startOaDimission(getLoginUserId(), createReqVO));
    }

    @PostMapping("/completeTure")
    @Operation(summary = "审批同意离职流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:complete-true')")
    public CommonResult<Long> completeTrueOaDimission(@Valid @RequestBody BpmTaskApproveReqVO reqVO) {
        return success(oaDimissionService.completeTrueOaDimission(getLoginUserId(), reqVO));
    }

    @PostMapping("/completeFalse")
    @Operation(summary = "驳回或撤回离职流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:complete-false')")
    public CommonResult<Long> completeFalseOaDimission(@Valid @RequestBody BpmTaskApproveReqVO reqVO) {
        return success(oaDimissionService.completeFalseOaDimission(getLoginUserId(), reqVO));
    }

    @PostMapping("/updateComplete")
    @Operation(summary = "驳回和撤回后再次提交或关闭离职流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:update-complete')")
    public CommonResult<Long> updateCompleteOaDimission(@Valid @RequestBody OaDimissionSaveReqVO createReqVO) {
        return success(oaDimissionService.updateCompleteOaDimission(getLoginUserId(), createReqVO, createReqVO.getBpmTaskApproveReqVO()));
    }


    @PostMapping("/create")
    @Operation(summary = "创建离职流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-dimission:create')")
    public CommonResult<Long> createOaDimission(@Valid @RequestBody OaDimissionSaveReqVO createReqVO) {
        return success(oaDimissionService.createOaDimission(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新离职流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-dimission:update')")
    public CommonResult<Boolean> updateOaDimission(@Valid @RequestBody OaDimissionSaveReqVO updateReqVO) {
        oaDimissionService.updateOaDimission(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除离职流程信息")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('bpm:oa-dimission:delete')")
    public CommonResult<Boolean> deleteOaDimission(@RequestParam("id") Long id) {
        oaDimissionService.deleteOaDimission(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得离职流程信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('bpm:oa-dimission:query')")
    public CommonResult<OaDimissionRespVO> getOaDimission(@RequestParam("id") Long id) {
        OaDimissionDO oaDimission = oaDimissionService.getOaDimission(id);
        return success(BeanUtils.toBean(oaDimission, OaDimissionRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得离职流程信息分页")
    @PreAuthorize("@ss.hasPermission('bpm:oa-dimission:query')")
    public CommonResult<PageResult<OaDimissionRespVO>> getOaDimissionPage(@Valid OaDimissionPageReqVO pageReqVO) {
        PageResult<OaDimissionDO> pageResult = oaDimissionService.getOaDimissionPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, OaDimissionRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出离职流程信息 Excel")
    @PreAuthorize("@ss.hasPermission('bpm:oa-dimission:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportOaDimissionExcel(@Valid OaDimissionPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<OaDimissionDO> list = oaDimissionService.getOaDimissionPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "离职流程信息.xls", "数据", OaDimissionRespVO.class,
                        BeanUtils.toBean(list, OaDimissionRespVO.class));
    }

}