package cn.iocoder.yudao.module.bpm.controller.admin.oa.entry;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.entry.vo.OaEntryPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.entry.vo.OaEntryRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.entry.vo.OaEntrySaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.entry.OaEntryDO;
import cn.iocoder.yudao.module.bpm.service.oa.entry.OaEntryService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;


@Tag(name = "管理后台 - 入职流程信息")
@RestController
@RequestMapping("/bpm/oa-entry")
@Validated
public class OaEntryController {

    @Resource
    private OaEntryService oaEntryService;

    @PostMapping("/start")
    @Operation(summary = "提交或暂存入职流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:start')")
    public CommonResult<Integer> startOaEntry(@Valid @RequestBody OaEntrySaveReqVO createReqVO) {
        return success(oaEntryService.startOaEntry(getLoginUserId(), createReqVO));
    }

    @PostMapping("/completeTure")
    @Operation(summary = "审批同意入职流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:complete-true')")
    public CommonResult<Integer> completeTrueOaEntry(@Valid @RequestBody BpmTaskApproveReqVO reqVO) {
        return success(oaEntryService.completeTrueOaEntry(getLoginUserId(), reqVO));
    }

    @PostMapping("/completeFalse")
    @Operation(summary = "驳回或撤回入职流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:complete-false')")
    public CommonResult<Integer> completeFalseOaEntry(@Valid @RequestBody BpmTaskApproveReqVO reqVO) {
        return success(oaEntryService.completeFalseOaEntry(getLoginUserId(), reqVO));
    }

    @PostMapping("/updateComplete")
    @Operation(summary = "驳回和撤回后再次提交或关闭入职流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:update-complete')")
    public CommonResult<Integer> updateCompleteOaEntry(@Valid @RequestBody OaEntrySaveReqVO createReqVO) {
        return success(oaEntryService.updateCompleteOaEntry(getLoginUserId(), createReqVO, createReqVO.getBpmTaskApproveReqVO()));
    }

//    @PostMapping("/return")
//    @Operation(summary = "撤回入职流程信息")
//    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:start')")
//    public CommonResult<Integer> returnCompelete(@Valid @RequestBody OaEntrySaveReqVO createReqVO,@Valid @RequestBody BpmTaskApproveReqVO reqVO) {
//        return success(oaEntryService.completeOaEntry(getLoginUserId(),createReqVO,reqVO));
//    }

//    @PutMapping("/update")
//    @Operation(summary = "更新入职流程信息")
//    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:update')")
//    public CommonResult<Boolean> updateOaEntry(@Valid @RequestBody OaEntrySaveReqVO updateReqVO) {
//        oaEntryService.updateOaEntry(updateReqVO);
//        return success(true);
//    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除入职流程信息")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:delete')")
    public CommonResult<Boolean> deleteOaEntry(@RequestParam("id") Integer id) {
        oaEntryService.deleteOaEntry(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得入职流程信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:query')")
    public CommonResult<OaEntryRespVO> getOaEntry(@RequestParam("id") Integer id) {
        OaEntryDO oaEntry = oaEntryService.getOaEntry(id);
        return success(BeanUtils.toBean(oaEntry, OaEntryRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得入职流程信息分页")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:query')")
    public CommonResult<PageResult<OaEntryRespVO>> getOaEntryPage(@Valid OaEntryPageReqVO pageReqVO) {
        PageResult<OaEntryDO> pageResult = oaEntryService.getOaEntryPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, OaEntryRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出入职流程信息 Excel")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportOaEntryExcel(@Valid OaEntryPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<OaEntryDO> list = oaEntryService.getOaEntryPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "入职流程信息.xls", "数据", OaEntryRespVO.class,
                        BeanUtils.toBean(list, OaEntryRespVO.class));
    }

}