package cn.iocoder.yudao.module.bpm.controller.admin.oa.holiday;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.holiday.vo.OaHolidayPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.holiday.vo.OaHolidayRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.holiday.vo.OaHolidaySaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.holiday.OaHolidayDO;
import cn.iocoder.yudao.module.bpm.service.oa.holiday.OaHolidayService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

@Tag(name = "管理后台 - 请假流程信息")
@RestController
@RequestMapping("/bpm/oa-holiday")
@Validated
public class OaHolidayController {

    @Resource
    private OaHolidayService oaHolidayService;

    @PostMapping("/start")
    @Operation(summary = "提交或暂存请假流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-Holiday:start')")
    public CommonResult<Long> startOaHoliday(@Valid @RequestBody OaHolidaySaveReqVO createReqVO) {
        return success(oaHolidayService.startOaHoliday(getLoginUserId(), createReqVO));
    }

    @PostMapping("/completeTure")
    @Operation(summary = "审批同意请假流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:complete-true')")
    public CommonResult<Long> completeTrueOaHoliday(@Valid @RequestBody BpmTaskApproveReqVO reqVO) {
        return success(oaHolidayService.completeTrueOaHoliday(getLoginUserId(), reqVO));
    }

    @PostMapping("/completeFalse")
    @Operation(summary = "驳回或撤回请假流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:complete-false')")
    public CommonResult<Long> completeFalseOaHoliday(@Valid @RequestBody BpmTaskApproveReqVO reqVO) {
        return success(oaHolidayService.completeFalseOaHoliday(getLoginUserId(), reqVO));
    }

    @PostMapping("/updateComplete")
    @Operation(summary = "驳回和撤回后再次提交或关闭请假流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:update-complete')")
    public CommonResult<Long> updateCompleteOaHoliday(@Valid @RequestBody OaHolidaySaveReqVO createReqVO) {
        return success(oaHolidayService.updateCompleteOaHoliday(getLoginUserId(), createReqVO, createReqVO.getBpmTaskApproveReqVO()));
    }


    @PostMapping("/create")
    @Operation(summary = "创建请假流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-holiday:create')")
    public CommonResult<Long> createOaHoliday(@Valid @RequestBody OaHolidaySaveReqVO createReqVO) {
        return success(oaHolidayService.createOaHoliday(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新请假流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-holiday:update')")
    public CommonResult<Boolean> updateOaHoliday(@Valid @RequestBody OaHolidaySaveReqVO updateReqVO) {
        oaHolidayService.updateOaHoliday(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除请假流程信息")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('bpm:oa-holiday:delete')")
    public CommonResult<Boolean> deleteOaHoliday(@RequestParam("id") Long id) {
        oaHolidayService.deleteOaHoliday(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得请假流程信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('bpm:oa-holiday:query')")
    public CommonResult<OaHolidayRespVO> getOaHoliday(@RequestParam("id") Long id) {
        OaHolidayDO oaHoliday = oaHolidayService.getOaHoliday(id);
        return success(BeanUtils.toBean(oaHoliday, OaHolidayRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得请假流程信息分页")
    @PreAuthorize("@ss.hasPermission('bpm:oa-holiday:query')")
    public CommonResult<PageResult<OaHolidayRespVO>> getOaHolidayPage(@Valid OaHolidayPageReqVO pageReqVO) {
        PageResult<OaHolidayDO> pageResult = oaHolidayService.getOaHolidayPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, OaHolidayRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出请假流程信息 Excel")
    @PreAuthorize("@ss.hasPermission('bpm:oa-holiday:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportOaHolidayExcel(@Valid OaHolidayPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<OaHolidayDO> list = oaHolidayService.getOaHolidayPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "请假流程信息.xls", "数据", OaHolidayRespVO.class,
                        BeanUtils.toBean(list, OaHolidayRespVO.class));
    }

}