package cn.iocoder.yudao.module.bpm.controller.admin.oa.leave;

import cn.iocoder.yudao.module.bpm.controller.admin.oa.leave.vo.OaLeavePageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.leave.vo.OaLeaveRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.leave.vo.OaLeaveSaveReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.leave.OaLeaveDO;
import cn.iocoder.yudao.module.bpm.service.oa.leave.OaLeaveService;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.*;


@Tag(name = "管理后台 - 请假流程信息")
@RestController
@RequestMapping("/bpm/oa-leave")
@Validated
public class OaLeaveController {

    @Resource
    private OaLeaveService oaLeaveService;

    @PostMapping("/create")
    @Operation(summary = "创建请假流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-leave:create')")
    public CommonResult<Integer> createOaLeave(@Valid @RequestBody OaLeaveSaveReqVO createReqVO) {
        return success(oaLeaveService.createOaLeave(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新请假流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-leave:update')")
    public CommonResult<Boolean> updateOaLeave(@Valid @RequestBody OaLeaveSaveReqVO updateReqVO) {
        oaLeaveService.updateOaLeave(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除请假流程信息")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('bpm:oa-leave:delete')")
    public CommonResult<Boolean> deleteOaLeave(@RequestParam("id") Integer id) {
        oaLeaveService.deleteOaLeave(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得请假流程信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('bpm:oa-leave:query')")
    public CommonResult<OaLeaveRespVO> getOaLeave(@RequestParam("id") Integer id) {
        OaLeaveDO oaLeave = oaLeaveService.getOaLeave(id);
        return success(BeanUtils.toBean(oaLeave, OaLeaveRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得请假流程信息分页")
    @PreAuthorize("@ss.hasPermission('bpm:oa-leave:query')")
    public CommonResult<PageResult<OaLeaveRespVO>> getOaLeavePage(@Valid OaLeavePageReqVO pageReqVO) {
        PageResult<OaLeaveDO> pageResult = oaLeaveService.getOaLeavePage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, OaLeaveRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出请假流程信息 Excel")
    @PreAuthorize("@ss.hasPermission('bpm:oa-leave:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportOaLeaveExcel(@Valid OaLeavePageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<OaLeaveDO> list = oaLeaveService.getOaLeavePage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "请假流程信息.xls", "数据", OaLeaveRespVO.class,
                        BeanUtils.toBean(list, OaLeaveRespVO.class));
    }

}