package cn.iocoder.yudao.module.bpm.controller.admin.oa.renew;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.renew.vo.OaRenewPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.renew.vo.OaRenewRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.renew.vo.OaRenewSaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.renew.OaRenewDO;
import cn.iocoder.yudao.module.bpm.service.oa.renew.OaRenewService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

@Tag(name = "管理后台 - 合同续签流程信息")
@RestController
@RequestMapping("/bpm/oa-renew")
@Validated
public class OaRenewController {

    @Resource
    private OaRenewService oaRenewService;

    @PostMapping("/start")
    @Operation(summary = "提交或暂存合同续签流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-Renew:start')")
    public CommonResult<Long> startOaRenew(@Valid @RequestBody OaRenewSaveReqVO createReqVO) {
        return success(oaRenewService.startOaRenew(getLoginUserId(), createReqVO));
    }

    @PostMapping("/completeTure")
    @Operation(summary = "审批同意合同续签流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:complete-true')")
    public CommonResult<Long> completeTrueOaRenew(@Valid @RequestBody BpmTaskApproveReqVO reqVO) {
        return success(oaRenewService.completeTrueOaRenew(getLoginUserId(), reqVO));
    }

    @PostMapping("/completeFalse")
    @Operation(summary = "驳回或撤回合同续签流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:complete-false')")
    public CommonResult<Long> completeFalseOaRenew(@Valid @RequestBody BpmTaskApproveReqVO reqVO) {
        return success(oaRenewService.completeFalseOaRenew(getLoginUserId(), reqVO));
    }

    @PostMapping("/updateComplete")
    @Operation(summary = "驳回和撤回后再次提交或关闭合同续签流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-entry:update-complete')")
    public CommonResult<Long> updateCompleteOaRenew(@Valid @RequestBody OaRenewSaveReqVO createReqVO) {
        return success(oaRenewService.updateCompleteOaRenew(getLoginUserId(), createReqVO, createReqVO.getBpmTaskApproveReqVO()));
    }


    @PostMapping("/create")
    @Operation(summary = "创建合同续签流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-renew:create')")
    public CommonResult<Long> createOaRenew(@Valid @RequestBody OaRenewSaveReqVO createReqVO) {
        return success(oaRenewService.createOaRenew(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新合同续签流程信息")
    @PreAuthorize("@ss.hasPermission('bpm:oa-renew:update')")
    public CommonResult<Boolean> updateOaRenew(@Valid @RequestBody OaRenewSaveReqVO updateReqVO) {
        oaRenewService.updateOaRenew(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除合同续签流程信息")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('bpm:oa-renew:delete')")
    public CommonResult<Boolean> deleteOaRenew(@RequestParam("id") Long id) {
        oaRenewService.deleteOaRenew(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得合同续签流程信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('bpm:oa-renew:query')")
    public CommonResult<OaRenewRespVO> getOaRenew(@RequestParam("id") Long id) {
        OaRenewDO oaRenew = oaRenewService.getOaRenew(id);
        return success(BeanUtils.toBean(oaRenew, OaRenewRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得合同续签流程信息分页")
    @PreAuthorize("@ss.hasPermission('bpm:oa-renew:query')")
    public CommonResult<PageResult<OaRenewRespVO>> getOaRenewPage(@Valid OaRenewPageReqVO pageReqVO) {
        PageResult<OaRenewDO> pageResult = oaRenewService.getOaRenewPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, OaRenewRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出合同续签流程信息 Excel")
    @PreAuthorize("@ss.hasPermission('bpm:oa-renew:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportOaRenewExcel(@Valid OaRenewPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<OaRenewDO> list = oaRenewService.getOaRenewPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "合同续签流程信息.xls", "数据", OaRenewRespVO.class,
                        BeanUtils.toBean(list, OaRenewRespVO.class));
    }

}