package cn.iocoder.yudao.module.bpm.service.oa.dimission;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.dimission.vo.OaDimissionPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.dimission.vo.OaDimissionSaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskReturnReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.dimission.OaDimissionDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.dimission.OaDimissionMapper;
import cn.iocoder.yudao.module.bpm.service.task.BpmTaskService;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.flowable.engine.TaskService;
import org.flowable.task.api.Task;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_DIMISSION_NOT_EXISTS;

/**
 * 离职流程信息 Service 实现类
 *
 * @author dp
 */
@Service
@Validated
public class OaDimissionServiceImpl extends ServiceImpl<OaDimissionMapper, OaDimissionDO> implements OaDimissionService {


    /**
     * OA 离职对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_dimission";


    @Resource
    private OaDimissionMapper oaDimissionMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Resource
    private BpmTaskService bpmTaskService;

    @Resource
    private TaskService taskService;

    @Override
    @Transactional
    public Long startOaDimission(Long userId, OaDimissionSaveReqVO createReqVO) {
        //判断是否已经发起流程
        Boolean auditPass = createReqVO.getAuditPass();
        // 创建单据uuid
        String uuid = IdUtil.fastSimpleUUID();
        OaDimissionDO oaDimission = BeanUtils.toBean(createReqVO, OaDimissionDO.class);
        if (StringUtils.isBlank(oaDimission.getDimissionId())) {
            oaDimission.setDimissionId(uuid);
        }
        // 保存或更新表单信息
        this.saveOrUpdate(oaDimission);

        // 提交
        if (auditPass) {
            // 发起 BPM 流程
            Map<String, Object> processInstanceVariables = new HashMap<>();
            processInstanceVariables.put("auditPass", auditPass);
            //添加审批人信息
            Map<String, List<Long>> startUserSelectAssignees = new HashMap<>();
            startUserSelectAssignees.put("approver", createReqVO.getStartUserSelectAssignees());
            String processInstanceId = processInstanceApi
                    .createProcessInstance(userId,
                            new BpmProcessInstanceCreateReqDTO()
                                    .setProcessDefinitionKey(PROCESS_KEY)
                                    .setVariables(processInstanceVariables)
                                    .setBusinessKey(String.valueOf(oaDimission.getId()))
                                    .setStartUserSelectAssignees(startUserSelectAssignees));

            //获取下一个审批人
            Task task = taskService.createTaskQuery().processInstanceId(processInstanceId).singleResult();

            // 将工作流的编号，单据状态 最后一次审批时间 当前审批人  更新到 OA 转正中
            oaDimissionMapper.updateById(new OaDimissionDO()
                    .setId(oaDimission.getId())
                    .setProcInstId(processInstanceId)
                    .setAuditStatus("1")
                    .setDimissionId(uuid)
                    .setFinalAuditDate(LocalDateTime.now())
                    .setCurrentAuditEmployeeId(Long.valueOf(task.getAssignee())));

            // TODO 发送通知

        }else {
            // 暂存，不发起流程
            // 将单据状态 更新到 OA转正中
            oaDimissionMapper.updateById(new OaDimissionDO()
                    .setId(oaDimission.getId())
                    .setAuditStatus("0"));
        }

        // TODO 保存或更新附件uuid
        if (CollectionUtil.isNotEmpty(createReqVO.getFileIdList())) {
        }

        // 返回
        return oaDimission.getId();
    }
    @Override
    public Long completeTrueOaDimission(Long userId, BpmTaskApproveReqVO reqVO) {
        Task currentTask = bpmTaskService.getTask(reqVO.getId());
        // 审批同意
        bpmTaskService.approveTask(userId, reqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        if (nextTask != null) {
            LambdaUpdateWrapper<OaDimissionDO> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.set(OaDimissionDO::getAuditStatus, "2")
                    .set(OaDimissionDO::getCurrentAuditEmployeeId, nextTask.getAssignee())
                    .set(OaDimissionDO::getFinalAuditDate, new Date())
                    .eq(OaDimissionDO::getProcInstId, currentTask.getProcessInstanceId());
            oaDimissionMapper.update(updateWrapper);
        } else {
            LambdaUpdateWrapper<OaDimissionDO> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.set(OaDimissionDO::getAuditStatus, "3")
                    .set(OaDimissionDO::getCurrentAuditEmployeeId, null)
                    .set(OaDimissionDO::getFinalAuditDate, new Date())
                    .eq(OaDimissionDO::getProcInstId, currentTask.getProcessInstanceId());
            oaDimissionMapper.update(updateWrapper);

            // TODO 复制业务单据信息到业务模块单据表

        }
        // 发送通知

        // 返回
        return 1L;
    }

    @Override
    public Long completeFalseOaDimission(Long userId, BpmTaskApproveReqVO reqVO) {

        Task currentTask = bpmTaskService.getTask(reqVO.getId());

        BpmTaskReturnReqVO reqVO2 = BeanUtils.toBean(reqVO, BpmTaskReturnReqVO.class);
        reqVO2.setTargetTaskDefinitionKey("modifyApply");
        bpmTaskService.returnTask(userId, reqVO2);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();

        // 更新单据状态，当前处理人，最后处理时间
        LambdaUpdateWrapper<OaDimissionDO> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(OaDimissionDO::getCurrentAuditEmployeeId, nextTask.getAssignee())
                .set(OaDimissionDO::getFinalAuditDate, new Date())
                .set(OaDimissionDO::getAuditStatus, "0")
                .eq(OaDimissionDO::getProcInstId, currentTask.getProcessInstanceId());
        oaDimissionMapper.update(updateWrapper);

        // 发送通知

        // 返回
        return 1L;
    }

    @Override
    public Long updateCompleteOaDimission(Long userId, OaDimissionSaveReqVO createReqVO, BpmTaskApproveReqVO reqVO) {
        // 判断是否已经发起流程
        Boolean auditPass = createReqVO.getAuditPass();

        OaDimissionDO oaDimission = BeanUtils.toBean(createReqVO, OaDimissionDO.class);

        Task currentTask = taskService.createTaskQuery().processInstanceId(oaDimission.getProcInstId()).singleResult();

        //审批通过
        bpmTaskService.approveTask(userId, reqVO);

        // 提交
        if (auditPass) {
            Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();

            oaDimission.setAuditStatus("1");
            oaDimission.setCurrentAuditEmployeeId(Long.valueOf(nextTask.getAssignee()));
            oaDimission.setFinalAuditDate(LocalDateTime.now());
            oaDimissionMapper.updateById(oaDimission);

        } else {
            OaDimissionDO oaDimissionClose = new OaDimissionDO();
            oaDimissionClose.setId(oaDimission.getId());
            oaDimissionClose.setAuditStatus("4");
            oaDimissionClose.setCurrentAuditEmployeeId(null);
            oaDimissionClose.setFinalAuditDate(LocalDateTime.now());
            oaDimissionMapper.updateById(oaDimissionClose);
        }

        return oaDimission.getId();
    }


    @Override
    public Long createOaDimission(OaDimissionSaveReqVO createReqVO) {
        // 插入
        OaDimissionDO oaDimission = BeanUtils.toBean(createReqVO, OaDimissionDO.class);
        oaDimissionMapper.insert(oaDimission);
        // 返回
        return oaDimission.getId();
    }

    @Override
    public void updateOaDimission(OaDimissionSaveReqVO updateReqVO) {
        // 校验存在
        validateOaDimissionExists(updateReqVO.getId());
        // 更新
        OaDimissionDO updateObj = BeanUtils.toBean(updateReqVO, OaDimissionDO.class);
        oaDimissionMapper.updateById(updateObj);
    }

    @Override
    public void deleteOaDimission(Long id) {
        // 校验存在
        validateOaDimissionExists(id);
        // 删除
        oaDimissionMapper.deleteById(id);
    }

    private void validateOaDimissionExists(Long id) {
        if (oaDimissionMapper.selectById(id) == null) {
            throw exception(OA_DIMISSION_NOT_EXISTS);
        }
    }

    @Override
    public OaDimissionDO getOaDimission(Long id) {
        return oaDimissionMapper.selectById(id);
    }

    @Override
    public PageResult<OaDimissionDO> getOaDimissionPage(OaDimissionPageReqVO pageReqVO) {
        return oaDimissionMapper.selectPage(pageReqVO);
    }

}