package cn.iocoder.yudao.module.bpm.service.oa.entry;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.entry.vo.OaEntryPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.entry.vo.OaEntrySaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.entry.OaEntryDO;
import com.baomidou.mybatisplus.extension.service.IService;

import javax.validation.Valid;

/**
 * 入职流程信息 Service 接口
 *
 * @author 芋道源码
 */
public interface OaEntryService extends IService<OaEntryDO> {

    /**
     * 发起入职流程信息
     *
     * @param userId
     * @param createReqVO 创建信息
     * @return 编号
     */
    Integer startOaEntry(Long userId, @Valid OaEntrySaveReqVO createReqVO);

    /**
     * 审批同意入职流程信息
     *
     * @param userId
     * @param reqVO  通过请求
     * @return 编号
     */
    Integer completeTrueOaEntry(Long userId, BpmTaskApproveReqVO reqVO);

    /**
     * 驳回或撤回入职流程信息
     *
     * @param userId
     * @param reqVO  驳回或撤回
     * @return 编号
     */
    Integer completeFalseOaEntry(Long userId, BpmTaskApproveReqVO reqVO);

    /**
     * 驳回和撤回后再次提交或关闭入职流程信息
     *
     * @param userId
     * @param createReqVO 创建信息
     * @param reqVO       通过请求
     * @return 编号
     */
    Integer updateCompleteOaEntry(Long userId, @Valid OaEntrySaveReqVO createReqVO, @Valid BpmTaskApproveReqVO reqVO);

    /**
     * 更新入职流程信息
     *
     * @param updateReqVO 更新信息
     */
    void updateOaEntry(@Valid OaEntrySaveReqVO updateReqVO);

    /**
     * 删除入职流程信息
     *
     * @param id 编号
     */
    void deleteOaEntry(Integer id);

    /**
     * 获得入职流程信息
     *
     * @param id 编号
     * @return 入职流程信息
     */
    OaEntryDO getOaEntry(Integer id);

    /**
     * 获得入职流程信息分页
     *
     * @param pageReqVO 分页查询
     * @return 入职流程信息分页
     */
    PageResult<OaEntryDO> getOaEntryPage(OaEntryPageReqVO pageReqVO);

}