package cn.iocoder.yudao.module.bpm.service.oa.entry;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.entry.vo.OaEntryPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.entry.vo.OaEntrySaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskReturnReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.entry.OaEntryDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.entry.OaEntryMapper;
import cn.iocoder.yudao.module.bpm.service.task.BpmTaskService;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.TaskService;
import org.flowable.task.api.Task;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_ENTRY_NOT_EXISTS;

/**
 * 入职流程信息 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class OaEntryServiceImpl extends ServiceImpl<OaEntryMapper, OaEntryDO> implements OaEntryService {

    /**
     * OA 入职对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_entry_workflow";

    @Resource
    private OaEntryMapper oaEntryMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Resource
    private BpmTaskService bpmTaskService;

    @Resource
    private TaskService taskService;

    @Autowired
    private RuntimeService runtimeService;

    @Override
    @Transactional
    public Integer startOaEntry(Long userId, OaEntrySaveReqVO createReqVO) {
        //判断是否已经发起流程
        Boolean auditPass = createReqVO.getAuditPass();
        // 创建单据uuid
        String uuid = IdUtil.fastSimpleUUID();
        OaEntryDO oaEntry = BeanUtils.toBean(createReqVO, OaEntryDO.class);
        if (StringUtils.isBlank(oaEntry.getEntryId())) {
            oaEntry.setEntryId(uuid);
        }
        // 保存或更新表单信息
        this.saveOrUpdate(oaEntry);

        // 提交
        if (auditPass) {
            // 发起 BPM 流程
            Map<String, Object> processInstanceVariables = new HashMap<>();
            processInstanceVariables.put("auditPass", auditPass);
            //添加审批人信息
            Map<String, List<Long>> startUserSelectAssignees = new HashMap<>();
            startUserSelectAssignees.put("approver", createReqVO.getStartUserSelectAssignees());
            String processInstanceId = processInstanceApi
                    .createProcessInstance(userId,
                            new BpmProcessInstanceCreateReqDTO()
                                    .setProcessDefinitionKey(PROCESS_KEY)
                                    .setVariables(processInstanceVariables)
                                    .setBusinessKey(String.valueOf(oaEntry.getId()))
                                    .setStartUserSelectAssignees(startUserSelectAssignees));

            //获取下一个审批人
            Task task = taskService.createTaskQuery().processInstanceId(processInstanceId).singleResult();

            // 将工作流的编号，单据状态 最后一次审批时间 当前审批人  更新到 OA 入职中
            oaEntryMapper.updateById(new OaEntryDO()
                    .setId(oaEntry.getId())
                    .setProcInstId(processInstanceId)
                    .setAuditStatus("1")
                    .setEntryId(uuid)
                    .setFinalAuditDate(new Date())
                    .setCurrentAuditEmployeeId(task.getAssignee()));

            // TODO 发送通知

        }else {
            // 暂存，不发起流程
            // 将单据状态 更新到 OA入职中
            oaEntryMapper.updateById(new OaEntryDO()
                    .setId(oaEntry.getId())
                    .setAuditStatus("0"));
        }

        // TODO 保存或更新附件uuid
        if (CollectionUtil.isNotEmpty(createReqVO.getFileIdList())) {
        }

        // 返回
        return oaEntry.getId();
    }

    @Override
    public Integer completeTrueOaEntry(Long userId, BpmTaskApproveReqVO reqVO) {
        Task currentTask = bpmTaskService.getTask(reqVO.getId());
        // 审批同意
        bpmTaskService.approveTask(userId, reqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        if (nextTask != null) {
            LambdaUpdateWrapper<OaEntryDO> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.set(OaEntryDO::getAuditStatus, "2")
                    .set(OaEntryDO::getCurrentAuditEmployeeId, nextTask.getAssignee())
                    .set(OaEntryDO::getFinalAuditDate, new Date())
                    .eq(OaEntryDO::getProcInstId, currentTask.getProcessInstanceId());
            oaEntryMapper.update(updateWrapper);
        } else {
            LambdaUpdateWrapper<OaEntryDO> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.set(OaEntryDO::getAuditStatus, "3")
                    .set(OaEntryDO::getCurrentAuditEmployeeId, null)
                    .set(OaEntryDO::getFinalAuditDate, new Date())
                    .eq(OaEntryDO::getProcInstId, currentTask.getProcessInstanceId());
            oaEntryMapper.update(updateWrapper);

            // TODO 复制业务单据信息到业务模块单据表

        }
        // 发送通知

        // 返回
        return 1;
    }

    @Override
    public Integer completeFalseOaEntry(Long userId, BpmTaskApproveReqVO reqVO) {

        Task currentTask = bpmTaskService.getTask(reqVO.getId());

        BpmTaskReturnReqVO reqVO2 = BeanUtils.toBean(reqVO, BpmTaskReturnReqVO.class);
        reqVO2.setTargetTaskDefinitionKey("modifyApply");
        bpmTaskService.returnTask(userId, reqVO2);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();

        // 更新单据状态，当前处理人，最后处理时间
        LambdaUpdateWrapper<OaEntryDO> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(OaEntryDO::getCurrentAuditEmployeeId, nextTask.getAssignee())
                .set(OaEntryDO::getFinalAuditDate, new Date())
                .set(OaEntryDO::getAuditStatus, "0")
                .eq(OaEntryDO::getProcInstId, currentTask.getProcessInstanceId());
        oaEntryMapper.update(updateWrapper);

        // 发送通知

        // 返回
        return 1;
    }

    @Override
    public Integer updateCompleteOaEntry(Long userId, OaEntrySaveReqVO createReqVO, BpmTaskApproveReqVO reqVO) {
        // 判断是否已经发起流程
        Boolean auditPass = createReqVO.getAuditPass();

        OaEntryDO oaEntry = BeanUtils.toBean(createReqVO, OaEntryDO.class);

        Task currentTask = taskService.createTaskQuery().processInstanceId(oaEntry.getProcInstId()).singleResult();

        //审批通过
        bpmTaskService.approveTask(userId, reqVO);

        // 提交
        if (auditPass) {
            Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();

            oaEntry.setAuditStatus("1");
            oaEntry.setCurrentAuditEmployeeId(nextTask.getAssignee());
            oaEntry.setFinalAuditDate(new Date());
            oaEntryMapper.updateById(oaEntry);

        } else {
            OaEntryDO oaEntryClose = new OaEntryDO();
            oaEntryClose.setId(oaEntry.getId());
            oaEntryClose.setAuditStatus("4");
            oaEntryClose.setCurrentAuditEmployeeId(null);
            oaEntryClose.setFinalAuditDate(new Date());
            oaEntryMapper.updateById(oaEntryClose);
        }

        return oaEntry.getId();
    }

    @Override
    public void updateOaEntry(OaEntrySaveReqVO updateReqVO) {
        // 校验存在
        validateOaEntryExists(updateReqVO.getId());
        // 更新
        OaEntryDO updateObj = BeanUtils.toBean(updateReqVO, OaEntryDO.class);
        oaEntryMapper.updateById(updateObj);
    }

    @Override
    public void deleteOaEntry(Integer id) {
        // 校验存在
        validateOaEntryExists(id);
        // 删除
        oaEntryMapper.deleteById(id);
    }

    private void validateOaEntryExists(Integer id) {
        if (oaEntryMapper.selectById(id) == null) {
            throw exception(OA_ENTRY_NOT_EXISTS);
        }
    }

    @Override
    public OaEntryDO getOaEntry(Integer id) {
        return oaEntryMapper.selectById(id);
    }

    @Override
    public PageResult<OaEntryDO> getOaEntryPage(OaEntryPageReqVO pageReqVO) {
        return oaEntryMapper.selectPage(pageReqVO);
    }

}