package cn.iocoder.yudao.module.bpm.service.oa.holiday;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.holiday.vo.OaHolidayPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.holiday.vo.OaHolidaySaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskReturnReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.holiday.OaHolidayDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.holiday.OaHolidayMapper;
import cn.iocoder.yudao.module.bpm.service.task.BpmTaskService;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.flowable.engine.TaskService;
import org.flowable.task.api.Task;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_HOLIDAY_NOT_EXISTS;

/**
 * 请假流程信息 Service 实现类
 *
 * @author dp
 */
@Service
@Validated
public class OaHolidayServiceImpl extends ServiceImpl<OaHolidayMapper, OaHolidayDO> implements OaHolidayService {


    /**
     * OA 离职对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_holiday";


    @Resource
    private OaHolidayMapper oaHolidayMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Resource
    private BpmTaskService bpmTaskService;

    @Resource
    private TaskService taskService;

    @Override
    @Transactional
    public Long startOaHoliday(Long userId, OaHolidaySaveReqVO createReqVO) {
        //判断是否已经发起流程
        Boolean auditPass = createReqVO.getAuditPass();
        // 创建单据uuid
        String uuid = IdUtil.fastSimpleUUID();
        OaHolidayDO oaHoliday = BeanUtils.toBean(createReqVO, OaHolidayDO.class);
        if (StringUtils.isBlank(oaHoliday.getHolidayId())) {
            oaHoliday.setHolidayId(uuid);
        }
        // 保存或更新表单信息
        this.saveOrUpdate(oaHoliday);

        // 提交
        if (auditPass) {
            // 发起 BPM 流程
            Map<String, Object> processInstanceVariables = new HashMap<>();
            processInstanceVariables.put("auditPass", auditPass);
            //添加审批人信息
            Map<String, List<Long>> startUserSelectAssignees = new HashMap<>();
            startUserSelectAssignees.put("approver", createReqVO.getStartUserSelectAssignees());
            String processInstanceId = processInstanceApi
                    .createProcessInstance(userId,
                            new BpmProcessInstanceCreateReqDTO()
                                    .setProcessDefinitionKey(PROCESS_KEY)
                                    .setVariables(processInstanceVariables)
                                    .setBusinessKey(String.valueOf(oaHoliday.getId()))
                                    .setStartUserSelectAssignees(startUserSelectAssignees));

            //获取下一个审批人
            Task task = taskService.createTaskQuery().processInstanceId(processInstanceId).singleResult();

            // 将工作流的编号，单据状态 最后一次审批时间 当前审批人  更新到 OA 转正中
            oaHolidayMapper.updateById(new OaHolidayDO()
                    .setId(oaHoliday.getId())
                    .setProcInstId(processInstanceId)
                    .setAuditStatus("1")
                    .setHolidayId(uuid)
                    .setFinalAuditDate(LocalDateTime.now())
                    .setCurrentAuditEmployeeId(Long.valueOf(task.getAssignee())));

            // TODO 发送通知

        }else {
            // 暂存，不发起流程
            // 将单据状态 更新到 OA转正中
            oaHolidayMapper.updateById(new OaHolidayDO()
                    .setId(oaHoliday.getId())
                    .setAuditStatus("0"));
        }

        // TODO 保存或更新附件uuid
        if (CollectionUtil.isNotEmpty(createReqVO.getFileIdList())) {
        }

        // 返回
        return oaHoliday.getId();
    }
    @Override
    public Long completeTrueOaHoliday(Long userId, BpmTaskApproveReqVO reqVO) {
        Task currentTask = bpmTaskService.getTask(reqVO.getId());
        // 审批同意
        bpmTaskService.approveTask(userId, reqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        if (nextTask != null) {
            LambdaUpdateWrapper<OaHolidayDO> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.set(OaHolidayDO::getAuditStatus, "2")
                    .set(OaHolidayDO::getCurrentAuditEmployeeId, nextTask.getAssignee())
                    .set(OaHolidayDO::getFinalAuditDate, new Date())
                    .eq(OaHolidayDO::getProcInstId, currentTask.getProcessInstanceId());
            oaHolidayMapper.update(updateWrapper);
        } else {
            LambdaUpdateWrapper<OaHolidayDO> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.set(OaHolidayDO::getAuditStatus, "3")
                    .set(OaHolidayDO::getCurrentAuditEmployeeId, null)
                    .set(OaHolidayDO::getFinalAuditDate, new Date())
                    .eq(OaHolidayDO::getProcInstId, currentTask.getProcessInstanceId());
            oaHolidayMapper.update(updateWrapper);

            // TODO 复制业务单据信息到业务模块单据表

        }
        // 发送通知

        // 返回
        return 1L;
    }

    @Override
    public Long completeFalseOaHoliday(Long userId, BpmTaskApproveReqVO reqVO) {

        Task currentTask = bpmTaskService.getTask(reqVO.getId());

        BpmTaskReturnReqVO reqVO2 = BeanUtils.toBean(reqVO, BpmTaskReturnReqVO.class);
        reqVO2.setTargetTaskDefinitionKey("modifyApply");
        bpmTaskService.returnTask(userId, reqVO2);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();

        // 更新单据状态，当前处理人，最后处理时间
        LambdaUpdateWrapper<OaHolidayDO> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(OaHolidayDO::getCurrentAuditEmployeeId, nextTask.getAssignee())
                .set(OaHolidayDO::getFinalAuditDate, new Date())
                .set(OaHolidayDO::getAuditStatus, "0")
                .eq(OaHolidayDO::getProcInstId, currentTask.getProcessInstanceId());
        oaHolidayMapper.update(updateWrapper);

        // 发送通知

        // 返回
        return 1L;
    }

    @Override
    public Long updateCompleteOaHoliday(Long userId, OaHolidaySaveReqVO createReqVO, BpmTaskApproveReqVO reqVO) {
        // 判断是否已经发起流程
        Boolean auditPass = createReqVO.getAuditPass();

        OaHolidayDO oaHoliday = BeanUtils.toBean(createReqVO, OaHolidayDO.class);

        Task currentTask = taskService.createTaskQuery().processInstanceId(oaHoliday.getProcInstId()).singleResult();

        //审批通过
        bpmTaskService.approveTask(userId, reqVO);

        // 提交
        if (auditPass) {
            Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();

            oaHoliday.setAuditStatus("1");
            oaHoliday.setCurrentAuditEmployeeId(Long.valueOf(nextTask.getAssignee()));
            oaHoliday.setFinalAuditDate(LocalDateTime.now());
            oaHolidayMapper.updateById(oaHoliday);

        } else {
            OaHolidayDO oaHolidayClose = new OaHolidayDO();
            oaHolidayClose.setId(oaHoliday.getId());
            oaHolidayClose.setAuditStatus("4");
            oaHolidayClose.setCurrentAuditEmployeeId(null);
            oaHolidayClose.setFinalAuditDate(LocalDateTime.now());
            oaHolidayMapper.updateById(oaHolidayClose);
        }

        return oaHoliday.getId();
    }



    @Override
    public Long createOaHoliday(OaHolidaySaveReqVO createReqVO) {
        // 插入
        OaHolidayDO oaHoliday = BeanUtils.toBean(createReqVO, OaHolidayDO.class);
        oaHolidayMapper.insert(oaHoliday);
        // 返回
        return oaHoliday.getId();
    }

    @Override
    public void updateOaHoliday(OaHolidaySaveReqVO updateReqVO) {
        // 校验存在
        validateOaHolidayExists(updateReqVO.getId());
        // 更新
        OaHolidayDO updateObj = BeanUtils.toBean(updateReqVO, OaHolidayDO.class);
        oaHolidayMapper.updateById(updateObj);
    }

    @Override
    public void deleteOaHoliday(Long id) {
        // 校验存在
        validateOaHolidayExists(id);
        // 删除
        oaHolidayMapper.deleteById(id);
    }

    private void validateOaHolidayExists(Long id) {
        if (oaHolidayMapper.selectById(id) == null) {
            throw exception(OA_HOLIDAY_NOT_EXISTS);
        }
    }

    @Override
    public OaHolidayDO getOaHoliday(Long id) {
        return oaHolidayMapper.selectById(id);
    }

    @Override
    public PageResult<OaHolidayDO> getOaHolidayPage(OaHolidayPageReqVO pageReqVO) {
        return oaHolidayMapper.selectPage(pageReqVO);
    }

}