package cn.iocoder.yudao.module.bpm.service.oa.leave;

import cn.iocoder.yudao.module.bpm.controller.admin.oa.leave.vo.OaLeavePageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.leave.vo.OaLeaveSaveReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.leave.OaLeaveDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.leave.OaLeaveMapper;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;



import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.*;

/**
 * 请假流程信息 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class OaLeaveServiceImpl implements OaLeaveService {

    @Resource
    private OaLeaveMapper oaLeaveMapper;

    @Override
    public Integer createOaLeave(OaLeaveSaveReqVO createReqVO) {
        // 插入
        OaLeaveDO oaLeave = BeanUtils.toBean(createReqVO, OaLeaveDO.class);
        oaLeaveMapper.insert(oaLeave);
        // 返回
        return oaLeave.getId();
    }

    @Override
    public void updateOaLeave(OaLeaveSaveReqVO updateReqVO) {
        // 校验存在
        validateOaLeaveExists(updateReqVO.getId());
        // 更新
        OaLeaveDO updateObj = BeanUtils.toBean(updateReqVO, OaLeaveDO.class);
        oaLeaveMapper.updateById(updateObj);
    }

    @Override
    public void deleteOaLeave(Integer id) {
        // 校验存在
        validateOaLeaveExists(id);
        // 删除
        oaLeaveMapper.deleteById(id);
    }

    private void validateOaLeaveExists(Integer id) {
        if (oaLeaveMapper.selectById(id) == null) {
            throw exception(OA_LEAVE_NOT_EXISTS);
        }
    }

    @Override
    public OaLeaveDO getOaLeave(Integer id) {
        return oaLeaveMapper.selectById(id);
    }

    @Override
    public PageResult<OaLeaveDO> getOaLeavePage(OaLeavePageReqVO pageReqVO) {
        return oaLeaveMapper.selectPage(pageReqVO);
    }

}