package cn.iocoder.yudao.module.bpm.service.oa.renew;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.renew.vo.OaRenewPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.renew.vo.OaRenewSaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskReturnReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.renew.OaRenewDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.renew.OaRenewMapper;
import cn.iocoder.yudao.module.bpm.service.task.BpmTaskService;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.flowable.engine.TaskService;
import org.flowable.task.api.Task;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_RENEW_NOT_EXISTS;

/**
 * 合同续签流程信息 Service 实现类
 *
 * @author dp
 */
@Service
@Validated
public class OaRenewServiceImpl extends ServiceImpl<OaRenewMapper, OaRenewDO> implements OaRenewService {


    /**
     * OA 合同续签对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_renew";

    @Resource
    private OaRenewMapper oaRenewMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Resource
    private BpmTaskService bpmTaskService;

    @Resource
    private TaskService taskService;

    @Override
    @Transactional
    public Long startOaRenew(Long userId, OaRenewSaveReqVO createReqVO) {
        //判断是否已经发起流程
        Boolean auditPass = createReqVO.getAuditPass();
        // 创建单据uuid
        String uuid = IdUtil.fastSimpleUUID();
        OaRenewDO oaRenew = BeanUtils.toBean(createReqVO, OaRenewDO.class);
        if (StringUtils.isBlank(oaRenew.getRenewId())) {
            oaRenew.setRenewId(uuid);
        }
        // 保存或更新表单信息
        this.saveOrUpdate(oaRenew);

        // 提交
        if (auditPass) {
            // 发起 BPM 流程
            Map<String, Object> processInstanceVariables = new HashMap<>();
            processInstanceVariables.put("auditPass", auditPass);
            //添加审批人信息
            Map<String, List<Long>> startUserSelectAssignees = new HashMap<>();
            startUserSelectAssignees.put("approver", createReqVO.getStartUserSelectAssignees());
            String processInstanceId = processInstanceApi
                    .createProcessInstance(userId,
                            new BpmProcessInstanceCreateReqDTO()
                                    .setProcessDefinitionKey(PROCESS_KEY)
                                    .setVariables(processInstanceVariables)
                                    .setBusinessKey(String.valueOf(oaRenew.getId()))
                                    .setStartUserSelectAssignees(startUserSelectAssignees));

            //获取下一个审批人
            Task task = taskService.createTaskQuery().processInstanceId(processInstanceId).singleResult();

            // 将工作流的编号，单据状态 最后一次审批时间 当前审批人  更新到 OA 转正中
            oaRenewMapper.updateById(new OaRenewDO()
                    .setId(oaRenew.getId())
                    .setProcInstId(processInstanceId)
                    .setAuditStatus("1")
                    .setRenewId(uuid)
                    .setFinalAuditDate(LocalDateTime.now())
                    .setCurrentAuditEmployeeId(Long.valueOf(task.getAssignee())));

            // TODO 发送通知

        }else {
            // 暂存，不发起流程
            // 将单据状态 更新到 OA转正中
            oaRenewMapper.updateById(new OaRenewDO()
                    .setId(oaRenew.getId())
                    .setAuditStatus("0"));
        }

        // TODO 保存或更新附件uuid
        if (CollectionUtil.isNotEmpty(createReqVO.getFileIdList())) {
        }

        // 返回
        return oaRenew.getId();
    }
    @Override
    public Long completeTrueOaRenew(Long userId, BpmTaskApproveReqVO reqVO) {
        Task currentTask = bpmTaskService.getTask(reqVO.getId());
        // 审批同意
        bpmTaskService.approveTask(userId, reqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        if (nextTask != null) {
            LambdaUpdateWrapper<OaRenewDO> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.set(OaRenewDO::getAuditStatus, "2")
                    .set(OaRenewDO::getCurrentAuditEmployeeId, nextTask.getAssignee())
                    .set(OaRenewDO::getFinalAuditDate, new Date())
                    .eq(OaRenewDO::getProcInstId, currentTask.getProcessInstanceId());
            oaRenewMapper.update(updateWrapper);
        } else {
            LambdaUpdateWrapper<OaRenewDO> updateWrapper = new LambdaUpdateWrapper<>();
            updateWrapper.set(OaRenewDO::getAuditStatus, "3")
                    .set(OaRenewDO::getCurrentAuditEmployeeId, null)
                    .set(OaRenewDO::getFinalAuditDate, new Date())
                    .eq(OaRenewDO::getProcInstId, currentTask.getProcessInstanceId());
            oaRenewMapper.update(updateWrapper);

            // TODO 复制业务单据信息到业务模块单据表

        }
        // 发送通知

        // 返回
        return 1L;
    }

    @Override
    public Long completeFalseOaRenew(Long userId, BpmTaskApproveReqVO reqVO) {

        Task currentTask = bpmTaskService.getTask(reqVO.getId());

        BpmTaskReturnReqVO reqVO2 = BeanUtils.toBean(reqVO, BpmTaskReturnReqVO.class);
        reqVO2.setTargetTaskDefinitionKey("modifyApply");
        bpmTaskService.returnTask(userId, reqVO2);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();

        // 更新单据状态，当前处理人，最后处理时间
        LambdaUpdateWrapper<OaRenewDO> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(OaRenewDO::getCurrentAuditEmployeeId, nextTask.getAssignee())
                .set(OaRenewDO::getFinalAuditDate, new Date())
                .set(OaRenewDO::getAuditStatus, "0")
                .eq(OaRenewDO::getProcInstId, currentTask.getProcessInstanceId());
        oaRenewMapper.update(updateWrapper);

        // 发送通知

        // 返回
        return 1L;
    }

    @Override
    public Long updateCompleteOaRenew(Long userId, OaRenewSaveReqVO createReqVO, BpmTaskApproveReqVO reqVO) {
        // 判断是否已经发起流程
        Boolean auditPass = createReqVO.getAuditPass();

        OaRenewDO oaRenew = BeanUtils.toBean(createReqVO, OaRenewDO.class);

        Task currentTask = taskService.createTaskQuery().processInstanceId(oaRenew.getProcInstId()).singleResult();

        //审批通过
        bpmTaskService.approveTask(userId, reqVO);

        // 提交
        if (auditPass) {
            Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();

            oaRenew.setAuditStatus("1");
            oaRenew.setCurrentAuditEmployeeId(Long.valueOf(nextTask.getAssignee()));
            oaRenew.setFinalAuditDate(LocalDateTime.now());
            oaRenewMapper.updateById(oaRenew);

        } else {
            OaRenewDO oaRenewClose = new OaRenewDO();
            oaRenewClose.setId(oaRenew.getId());
            oaRenewClose.setAuditStatus("4");
            oaRenewClose.setCurrentAuditEmployeeId(null);
            oaRenewClose.setFinalAuditDate(LocalDateTime.now());
            oaRenewMapper.updateById(oaRenewClose);
        }

        return oaRenew.getId();
    }


    @Override
    public Long createOaRenew(OaRenewSaveReqVO createReqVO) {
        // 插入
        OaRenewDO oaRenew = BeanUtils.toBean(createReqVO, OaRenewDO.class);
        oaRenewMapper.insert(oaRenew);
        // 返回
        return oaRenew.getId();
    }

    @Override
    public void updateOaRenew(OaRenewSaveReqVO updateReqVO) {
        // 校验存在
        validateOaRenewExists(updateReqVO.getId());
        // 更新
        OaRenewDO updateObj = BeanUtils.toBean(updateReqVO, OaRenewDO.class);
        oaRenewMapper.updateById(updateObj);
    }

    @Override
    public void deleteOaRenew(Long id) {
        // 校验存在
        validateOaRenewExists(id);
        // 删除
        oaRenewMapper.deleteById(id);
    }

    private void validateOaRenewExists(Long id) {
        if (oaRenewMapper.selectById(id) == null) {
            throw exception(OA_RENEW_NOT_EXISTS);
        }
    }

    @Override
    public OaRenewDO getOaRenew(Long id) {
        return oaRenewMapper.selectById(id);
    }

    @Override
    public PageResult<OaRenewDO> getOaRenewPage(OaRenewPageReqVO pageReqVO) {
        return oaRenewMapper.selectPage(pageReqVO);
    }

}