package cn.iocoder.yudao.module.bpm.service.oa.dimission;

import cn.iocoder.yudao.module.bpm.controller.admin.oa.dimission.vo.OaDimissionPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.dimission.vo.OaDimissionSaveReqVO;
import cn.iocoder.yudao.module.bpm.service.oa.dimission.OaDimissionServiceImpl;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;

import javax.annotation.Resource;

import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;

import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.dimission.OaDimissionDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.dimission.OaDimissionMapper;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import org.springframework.context.annotation.Import;

import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.*;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.*;
import static cn.iocoder.yudao.framework.common.util.date.LocalDateTimeUtils.*;
import static cn.iocoder.yudao.framework.common.util.object.ObjectUtils.*;
import static org.junit.jupiter.api.Assertions.*;

/**
 * {@link OaDimissionServiceImpl} 的单元测试类
 *
 * @author dp
 */
@Import(OaDimissionServiceImpl.class)
public class OaDimissionServiceImplTest extends BaseDbUnitTest {

    @Resource
    private OaDimissionServiceImpl oaDimissionService;

    @Resource
    private OaDimissionMapper oaDimissionMapper;

    @Test
    public void testCreateOaDimission_success() {
        // 准备参数
        OaDimissionSaveReqVO createReqVO = randomPojo(OaDimissionSaveReqVO.class).setId(null);

        // 调用
        Long oaDimissionId = oaDimissionService.createOaDimission(createReqVO);
        // 断言
        assertNotNull(oaDimissionId);
        // 校验记录的属性是否正确
        OaDimissionDO oaDimission = oaDimissionMapper.selectById(oaDimissionId);
        assertPojoEquals(createReqVO, oaDimission, "id");
    }

    @Test
    public void testUpdateOaDimission_success() {
        // mock 数据
        OaDimissionDO dbOaDimission = randomPojo(OaDimissionDO.class);
        oaDimissionMapper.insert(dbOaDimission);// @Sql: 先插入出一条存在的数据
        // 准备参数
        OaDimissionSaveReqVO updateReqVO = randomPojo(OaDimissionSaveReqVO.class, o -> {
            o.setId(dbOaDimission.getId()); // 设置更新的 ID
        });

        // 调用
        oaDimissionService.updateOaDimission(updateReqVO);
        // 校验是否更新正确
        OaDimissionDO oaDimission = oaDimissionMapper.selectById(updateReqVO.getId()); // 获取最新的
        assertPojoEquals(updateReqVO, oaDimission);
    }

    @Test
    public void testUpdateOaDimission_notExists() {
        // 准备参数
        OaDimissionSaveReqVO updateReqVO = randomPojo(OaDimissionSaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> oaDimissionService.updateOaDimission(updateReqVO), OA_DIMISSION_NOT_EXISTS);
    }

    @Test
    public void testDeleteOaDimission_success() {
        // mock 数据
        OaDimissionDO dbOaDimission = randomPojo(OaDimissionDO.class);
        oaDimissionMapper.insert(dbOaDimission);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbOaDimission.getId();

        // 调用
        oaDimissionService.deleteOaDimission(id);
       // 校验数据不存在了
       assertNull(oaDimissionMapper.selectById(id));
    }

    @Test
    public void testDeleteOaDimission_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> oaDimissionService.deleteOaDimission(id), OA_DIMISSION_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetOaDimissionPage() {
       // mock 数据
       OaDimissionDO dbOaDimission = randomPojo(OaDimissionDO.class, o -> { // 等会查询到
           o.setDimissionId(null);
           o.setEmployeeId(null);
           o.setEmployeeUuid(null);
           o.setEmployeeName(null);
           o.setUserId(null);
           o.setUserUuid(null);
           o.setDeptId(null);
           o.setDeptUuid(null);
           o.setPosition(null);
           o.setStartDate(null);
           o.setDimissionDate(null);
           o.setDimissionReason(null);
           o.setRemarks(null);
           o.setProcInstId(null);
           o.setAuditStatus(null);
           o.setCurrentAuditUserId(null);
           o.setCurrentAuditUserUuid(null);
           o.setCurrentAuditEmployeeId(null);
           o.setCurrentAuditEmployeeUuid(null);
           o.setFinalAuditDate(null);
           o.setInfoSource(null);
           o.setCreateTime(null);
       });
       oaDimissionMapper.insert(dbOaDimission);
       // 测试 dimissionId 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setDimissionId(null)));
       // 测试 employeeId 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setEmployeeId(null)));
       // 测试 employeeUuid 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setEmployeeUuid(null)));
       // 测试 employeeName 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setEmployeeName(null)));
       // 测试 userId 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setUserId(null)));
       // 测试 userUuid 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setUserUuid(null)));
       // 测试 deptId 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setDeptId(null)));
       // 测试 deptUuid 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setDeptUuid(null)));
       // 测试 position 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setPosition(null)));
       // 测试 startDate 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setStartDate(null)));
       // 测试 dimissionDate 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setDimissionDate(null)));
       // 测试 dimissionReason 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setDimissionReason(null)));
       // 测试 remarks 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setRemarks(null)));
       // 测试 procInstId 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setProcInstId(null)));
       // 测试 auditStatus 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setAuditStatus(null)));
       // 测试 currentAuditUserId 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setCurrentAuditUserId(null)));
       // 测试 currentAuditUserUuid 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setCurrentAuditUserUuid(null)));
       // 测试 currentAuditEmployeeId 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setCurrentAuditEmployeeId(null)));
       // 测试 currentAuditEmployeeUuid 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setCurrentAuditEmployeeUuid(null)));
       // 测试 finalAuditDate 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setFinalAuditDate(null)));
       // 测试 infoSource 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setInfoSource(null)));
       // 测试 createTime 不匹配
       oaDimissionMapper.insert(cloneIgnoreId(dbOaDimission, o -> o.setCreateTime(null)));
       // 准备参数
       OaDimissionPageReqVO reqVO = new OaDimissionPageReqVO();
       reqVO.setDimissionId(null);
       reqVO.setEmployeeId(null);
       reqVO.setEmployeeUuid(null);
       reqVO.setEmployeeName(null);
       reqVO.setUserId(null);
       reqVO.setUserUuid(null);
       reqVO.setDeptId(null);
       reqVO.setDeptUuid(null);
       reqVO.setPosition(null);
       reqVO.setStartDate(null);
       reqVO.setDimissionDate(null);
       reqVO.setDimissionReason(null);
       reqVO.setRemarks(null);
       reqVO.setProcInstId(null);
       reqVO.setAuditStatus(null);
       reqVO.setCurrentAuditUserId(null);
       reqVO.setCurrentAuditUserUuid(null);
       reqVO.setCurrentAuditEmployeeId(null);
       reqVO.setCurrentAuditEmployeeUuid(null);
       reqVO.setFinalAuditDate(buildBetweenTime(2023, 2, 1, 2023, 2, 28));
       reqVO.setInfoSource(null);
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<OaDimissionDO> pageResult = oaDimissionService.getOaDimissionPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbOaDimission, pageResult.getList().get(0));
    }

}