package cn.iocoder.yudao.module.bpm.service.oa.holiday;

import cn.iocoder.yudao.module.bpm.controller.admin.oa.holiday.vo.OaHolidayPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.holiday.vo.OaHolidaySaveReqVO;
import cn.iocoder.yudao.module.bpm.service.oa.holiday.OaHolidayServiceImpl;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;

import javax.annotation.Resource;

import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;

import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.holiday.OaHolidayDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.holiday.OaHolidayMapper;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import org.springframework.context.annotation.Import;

import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.*;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.*;
import static cn.iocoder.yudao.framework.common.util.date.LocalDateTimeUtils.*;
import static cn.iocoder.yudao.framework.common.util.object.ObjectUtils.*;
import static org.junit.jupiter.api.Assertions.*;

/**
 * {@link OaHolidayServiceImpl} 的单元测试类
 *
 * @author dp
 */
@Import(OaHolidayServiceImpl.class)
public class OaHolidayServiceImplTest extends BaseDbUnitTest {

    @Resource
    private OaHolidayServiceImpl oaHolidayService;

    @Resource
    private OaHolidayMapper oaHolidayMapper;

    @Test
    public void testCreateOaHoliday_success() {
        // 准备参数
        OaHolidaySaveReqVO createReqVO = randomPojo(OaHolidaySaveReqVO.class).setId(null);

        // 调用
        Long oaHolidayId = oaHolidayService.createOaHoliday(createReqVO);
        // 断言
        assertNotNull(oaHolidayId);
        // 校验记录的属性是否正确
        OaHolidayDO oaHoliday = oaHolidayMapper.selectById(oaHolidayId);
        assertPojoEquals(createReqVO, oaHoliday, "id");
    }

    @Test
    public void testUpdateOaHoliday_success() {
        // mock 数据
        OaHolidayDO dbOaHoliday = randomPojo(OaHolidayDO.class);
        oaHolidayMapper.insert(dbOaHoliday);// @Sql: 先插入出一条存在的数据
        // 准备参数
        OaHolidaySaveReqVO updateReqVO = randomPojo(OaHolidaySaveReqVO.class, o -> {
            o.setId(dbOaHoliday.getId()); // 设置更新的 ID
        });

        // 调用
        oaHolidayService.updateOaHoliday(updateReqVO);
        // 校验是否更新正确
        OaHolidayDO oaHoliday = oaHolidayMapper.selectById(updateReqVO.getId()); // 获取最新的
        assertPojoEquals(updateReqVO, oaHoliday);
    }

    @Test
    public void testUpdateOaHoliday_notExists() {
        // 准备参数
        OaHolidaySaveReqVO updateReqVO = randomPojo(OaHolidaySaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> oaHolidayService.updateOaHoliday(updateReqVO), OA_HOLIDAY_NOT_EXISTS);
    }

    @Test
    public void testDeleteOaHoliday_success() {
        // mock 数据
        OaHolidayDO dbOaHoliday = randomPojo(OaHolidayDO.class);
        oaHolidayMapper.insert(dbOaHoliday);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbOaHoliday.getId();

        // 调用
        oaHolidayService.deleteOaHoliday(id);
       // 校验数据不存在了
       assertNull(oaHolidayMapper.selectById(id));
    }

    @Test
    public void testDeleteOaHoliday_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> oaHolidayService.deleteOaHoliday(id), OA_HOLIDAY_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetOaHolidayPage() {
       // mock 数据
       OaHolidayDO dbOaHoliday = randomPojo(OaHolidayDO.class, o -> { // 等会查询到
           o.setHolidayId(null);
           o.setEmployeeId(null);
           o.setEmployeeUuid(null);
           o.setEmployeeName(null);
           o.setUserId(null);
           o.setUserUuid(null);
           o.setDeptId(null);
           o.setDeptUuid(null);
           o.setPosition(null);
           o.setHolidayType(null);
           o.setStartDate(null);
           o.setEndDate(null);
           o.setDay(null);
           o.setStartTime(null);
           o.setEndTime(null);
           o.setReason(null);
           o.setRemarks(null);
           o.setProcInstId(null);
           o.setAuditStatus(null);
           o.setCurrentAuditUserId(null);
           o.setCurrentAuditUserUuid(null);
           o.setCurrentAuditEmployeeId(null);
           o.setCurrentAuditEmployeeUuid(null);
           o.setFinalAuditDate(null);
           o.setInfoSource(null);
           o.setCreateTime(null);
       });
       oaHolidayMapper.insert(dbOaHoliday);
       // 测试 holidayId 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setHolidayId(null)));
       // 测试 employeeId 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setEmployeeId(null)));
       // 测试 employeeUuid 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setEmployeeUuid(null)));
       // 测试 employeeName 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setEmployeeName(null)));
       // 测试 userId 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setUserId(null)));
       // 测试 userUuid 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setUserUuid(null)));
       // 测试 deptId 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setDeptId(null)));
       // 测试 deptUuid 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setDeptUuid(null)));
       // 测试 position 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setPosition(null)));
       // 测试 holidayType 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setHolidayType(null)));
       // 测试 startDate 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setStartDate(null)));
       // 测试 endDate 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setEndDate(null)));
       // 测试 day 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setDay(null)));
       // 测试 startTime 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setStartTime(null)));
       // 测试 endTime 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setEndTime(null)));
       // 测试 reason 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setReason(null)));
       // 测试 remarks 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setRemarks(null)));
       // 测试 procInstId 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setProcInstId(null)));
       // 测试 auditStatus 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setAuditStatus(null)));
       // 测试 currentAuditUserId 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setCurrentAuditUserId(null)));
       // 测试 currentAuditUserUuid 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setCurrentAuditUserUuid(null)));
       // 测试 currentAuditEmployeeId 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setCurrentAuditEmployeeId(null)));
       // 测试 currentAuditEmployeeUuid 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setCurrentAuditEmployeeUuid(null)));
       // 测试 finalAuditDate 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setFinalAuditDate(null)));
       // 测试 infoSource 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setInfoSource(null)));
       // 测试 createTime 不匹配
       oaHolidayMapper.insert(cloneIgnoreId(dbOaHoliday, o -> o.setCreateTime(null)));
       // 准备参数
       OaHolidayPageReqVO reqVO = new OaHolidayPageReqVO();
       reqVO.setHolidayId(null);
       reqVO.setEmployeeId(null);
       reqVO.setEmployeeUuid(null);
       reqVO.setEmployeeName(null);
       reqVO.setUserId(null);
       reqVO.setUserUuid(null);
       reqVO.setDeptId(null);
       reqVO.setDeptUuid(null);
       reqVO.setPosition(null);
       reqVO.setHolidayType(null);
       reqVO.setStartDate(null);
       reqVO.setEndDate(null);
       reqVO.setDay(null);
       reqVO.setStartTime(null);
       reqVO.setEndTime(null);
       reqVO.setReason(null);
       reqVO.setRemarks(null);
       reqVO.setProcInstId(null);
       reqVO.setAuditStatus(null);
       reqVO.setCurrentAuditUserId(null);
       reqVO.setCurrentAuditUserUuid(null);
       reqVO.setCurrentAuditEmployeeId(null);
       reqVO.setCurrentAuditEmployeeUuid(null);
       reqVO.setFinalAuditDate(buildBetweenTime(2023, 2, 1, 2023, 2, 28));
       reqVO.setInfoSource(null);
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<OaHolidayDO> pageResult = oaHolidayService.getOaHolidayPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbOaHoliday, pageResult.getList().get(0));
    }

}