package cn.iocoder.yudao.module.infra.api.file;

import cn.iocoder.yudao.module.infra.api.file.dto.FileDTO;

import java.util.List;

/**
 * 文件 API 接口
 *
 * @author 芋道源码
 */
public interface FileApi {

    /**
     * 保存文件，并返回文件的访问路径
     *
     * @param content 文件内容
     * @return 文件路径
     */
    default String createFile(byte[] content) {
        return createFile(null, null, content);
    }

    /**
     * 保存文件，并返回文件的访问路径
     *
     * @param path 文件路径
     * @param content 文件内容
     * @return 文件路径
     */
    default String createFile(String path, byte[] content) {
        return createFile(null, path, content);
    }

    /**
     * 保存文件，并返回文件的访问路径
     *
     * @param name 文件名称
     * @param path 文件路径
     * @param content 文件内容
     * @return 文件路径
     */
    String createFile(String name, String path, byte[] content);

    /**
     * 保存文件，并返回文件的访问路径
     *
     * @param name    文件名称
     * @param path    文件路径
     * @param content 文件内容
     * @return 文件对象
     */
    FileDTO uploadFile(String name, String path, byte[] content);

    /**
     * 更新文件业务id
     *
     * @param fileIdList 文件id集合
     * @param serviceId  业务id
     */
    Integer updateFileBiz(List<Long> fileIdList, String serviceId);

    /**
     * 根据业务id获取文件信息列表
     *
     * @param universalId 业务id
     * @return
     */
    List<FileDTO> getFileDTOListByBiz(String universalId);
}
