package cn.iocoder.yudao.module.employee.controller.admin.info.vo;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.*;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.time.LocalDate;

@Schema(description = "管理后台 - 员工信息新增/修改 Request VO")
@Data
public class EmployeeInfoSaveReqVO {

    @Schema(description = "员工ID", requiredMode = Schema.RequiredMode.REQUIRED, example = "10984")
    private Long id;

    @Schema(description = "业务UUID", requiredMode = Schema.RequiredMode.REQUIRED, example = "1987")
    @NotEmpty(message = "业务UUID不能为空")
    private String infoId;

    @Schema(description = "姓名", requiredMode = Schema.RequiredMode.REQUIRED, example = "赵六")
    @NotEmpty(message = "姓名不能为空")
    private String name;

    @Schema(description = "部门ID", example = "22378")
    private Long deptId;

    @Schema(description = "部门名称", example = "芋艿")
    private String deptName;

    @Schema(description = "职位编号", example = "1056")
    private Long postId;

    @Schema(description = "职位名称")
    private String position;

    @Schema(description = "员工编号")
    private String employeeNumber;

    @Schema(description = "入职时间")
    private LocalDate entryDate;

    @Schema(description = "试用期到期时间")
    private LocalDate probationEndDate;

    @Schema(description = "离职时间")
    private LocalDate departureDate;

    @Schema(description = "离职原因", example = "不香")
    private String resignationReason;

    @Schema(description = "员工状态", requiredMode = Schema.RequiredMode.REQUIRED, example = "2")
    @NotEmpty(message = "员工状态不能为空")
    private String employeeStatus;

    @Schema(description = "性别")
    private String gender;

    @Schema(description = "电子邮箱")
    private String email;

    @Schema(description = "手机号")
    private String phone;

    @Schema(description = "身份证号")
    private String idCardNumber;

    @Schema(description = "身份证地址")
    private String idCardAddress;

    @Schema(description = "户口类型", example = "2")
    private String householdType;

    @Schema(description = "户口所在地")
    private String householdLocation;

    @Schema(description = "最高学历")
    private String educationLevel;

    @Schema(description = "毕业院校")
    private String graduationSchool;

    @Schema(description = "毕业时间")
    private LocalDate graduationDate;

    @Schema(description = "银行卡名称", example = "张三")
    private String bankCardName;

    @Schema(description = "银行卡卡号")
    private String bankCardNumber;

    @Schema(description = "年假基准天数")
    private Integer baseAnnualLeave;

    @Schema(description = "剩余年假天数")
    private Integer remainingAnnualLeave;

    @Schema(description = "出生日期")
    private LocalDate birthDate;

    @Schema(description = "婚姻状况", example = "2")
    private String marriageStatus;

    @Schema(description = "工作地点")
    private String workLocation;

    @Schema(description = "专业")
    private String major;

    @Schema(description = "参加工作时间")
    private LocalDate workStartDate;

    @Schema(description = "薪酬")
    private BigDecimal salary;

    @Schema(description = "岗位薪资")
    private BigDecimal positionSalary;

    @Schema(description = "项目津贴")
    private BigDecimal projectAllowance;

    @Schema(description = "特殊岗位津贴")
    private BigDecimal specialPositionAllowance;

    @Schema(description = "外籍津贴")
    private BigDecimal foreignAllowance;

    @Schema(description = "午餐补助")
    private BigDecimal lunchSubsidy;

    @Schema(description = "特别津贴")
    private BigDecimal specialAllowance;

    @Schema(description = "补贴")
    private BigDecimal subsidy;

    @Schema(description = "年终奖")
    private BigDecimal yearEndBonus;

    @Schema(description = "头像地址")
    private String avatar;

    @Schema(description = "状态", requiredMode = Schema.RequiredMode.REQUIRED, example = "2")
    @NotNull(message = "状态不能为空")
    private Integer status;

    @Schema(description = "租户编号", requiredMode = Schema.RequiredMode.REQUIRED, example = "27933")
    @NotNull(message = "租户编号不能为空")
    private Long tenantId;

    @Schema(description = "用户ID", requiredMode = Schema.RequiredMode.REQUIRED, example = "12259")
    @NotNull(message = "用户ID不能为空")
    private Long userId;

}