package cn.iocoder.yudao.module.employee.service.info;

import cn.hutool.core.util.IdUtil;
import cn.iocoder.yudao.framework.tenant.core.aop.TenantIgnore;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import cn.iocoder.yudao.module.employee.controller.admin.info.vo.*;
import cn.iocoder.yudao.module.employee.dal.dataobject.info.EmployeeInfoDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;

import cn.iocoder.yudao.module.employee.dal.mysql.info.EmployeeInfoMapper;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.employee.enums.ErrorCodeConstants.EMPLOYEE_INFO_NOT_EXISTS;

/**
 * 员工信息 Service 实现类
 *
 * @author zhaopq
 */
@Service
@Validated
public class EmployeeInfoServiceImpl implements EmployeeInfoService {

    @Resource
    private EmployeeInfoMapper infoMapper;
    @Resource
    private EmployeeInfoHistoryService employeeInfoHistoryService;

    @Override
    @TenantIgnore
    public Long createInfo(EmployeeInfoSaveReqVO createReqVO) {
        // 插入
        // 生成id
        createReqVO.setInfoId(IdUtil.fastSimpleUUID());
        EmployeeInfoDO info = BeanUtils.toBean(createReqVO, EmployeeInfoDO.class);
        infoMapper.insert(info);
        // 返回
        return info.getId();
    }

    @Override
    @TenantIgnore
    public void updateInfo(EmployeeInfoSaveReqVO updateReqVO) {
        // 校验存在
        validateInfoExists(updateReqVO.getId());
        // 更新
        EmployeeInfoDO updateObj = BeanUtils.toBean(updateReqVO, EmployeeInfoDO.class);
        // todo: 插入历史数据
        EmployeeInfoHistorySaveReqVO historySaveReqVO = BeanUtils.toBean(updateObj, EmployeeInfoHistorySaveReqVO.class);
        historySaveReqVO.setEmployeeId(updateObj.getId());
        employeeInfoHistoryService.createInfoHistory(historySaveReqVO);
        infoMapper.updateById(updateObj);
    }

    @Override
    @TenantIgnore
    public void deleteInfo(Long id) {
        // 校验存在
        validateInfoExists(id);
        // 删除
        infoMapper.deleteById(id);
    }

    @TenantIgnore
    private void validateInfoExists(Long id) {
        if (infoMapper.selectById(id) == null) {
            throw exception(EMPLOYEE_INFO_NOT_EXISTS);
        }
    }

    @Override
    @TenantIgnore
    public EmployeeInfoDO getInfo(Long id) {
        return infoMapper.selectById(id);
    }

    @Override
    @TenantIgnore
    public PageResult<EmployeeInfoDO> getInfoPage(EmployeeInfoPageReqVO pageReqVO) {
        return infoMapper.selectPage(pageReqVO);
    }

    @Override
    @TenantIgnore
    public EmployeeInfoDO getInfoByUserIdAndTenantId(Long userId, Long tenantId) {
        LambdaQueryWrapper<EmployeeInfoDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(EmployeeInfoDO::getUserId, userId)
                .eq(EmployeeInfoDO::getTenantId, tenantId);
        return infoMapper.selectOne(lambdaQueryWrapper);
    }

    @Override
    @TenantIgnore
    public EmployeeInfoDO getInfo(EmployeeInfoQueryReqVO queryReqVO) {
        return infoMapper.getInfo(queryReqVO);
    }

}