package cn.iocoder.yudao.module.relations.controller.admin.contractinfo;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.relations.controller.admin.contractinfo.vo.RelationsContractInfoPageReqVO;
import cn.iocoder.yudao.module.relations.controller.admin.contractinfo.vo.RelationsContractInfoRespVO;
import cn.iocoder.yudao.module.relations.controller.admin.contractinfo.vo.RelationsContractInfoSaveReqVO;
import cn.iocoder.yudao.module.relations.dal.dataobject.contractinfo.RelationsContractInfoDO;
import cn.iocoder.yudao.module.relations.service.contractinfo.RelationsContractInfoService;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.*;


@Tag(name = "管理后台 - 员工合同信息")
@RestController
@RequestMapping("/relations/contract-info")
@Validated
public class RelationsContractInfoController {

    @Resource
    private RelationsContractInfoService contractInfoService;

    @PostMapping("/create")
    @Operation(summary = "创建员工合同信息")
    @PreAuthorize("@ss.hasPermission('relations:contract-info:create')")
    public CommonResult<Long> createContractInfo(@Valid @RequestBody RelationsContractInfoSaveReqVO createReqVO) {
        return success(contractInfoService.createContractInfo(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新员工合同信息")
    @PreAuthorize("@ss.hasPermission('relations:contract-info:update')")
    public CommonResult<Boolean> updateContractInfo(@Valid @RequestBody RelationsContractInfoSaveReqVO updateReqVO) {
        contractInfoService.updateContractInfo(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除员工合同信息")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('relations:contract-info:delete')")
    public CommonResult<Boolean> deleteContractInfo(@RequestParam("id") Long id) {
        contractInfoService.deleteContractInfo(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得员工合同信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('relations:contract-info:query')")
    public CommonResult<RelationsContractInfoRespVO> getContractInfo(@RequestParam("id") Long id) {
        RelationsContractInfoDO contractInfo = contractInfoService.getContractInfo(id);
        return success(BeanUtils.toBean(contractInfo, RelationsContractInfoRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得员工合同信息分页")
    @PreAuthorize("@ss.hasPermission('relations:contract-info:query')")
    public CommonResult<PageResult<RelationsContractInfoRespVO>> getContractInfoPage(@Valid RelationsContractInfoPageReqVO pageReqVO) {
        PageResult<RelationsContractInfoDO> pageResult = contractInfoService.getContractInfoPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, RelationsContractInfoRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出员工合同信息 Excel")
    @PreAuthorize("@ss.hasPermission('relations:contract-info:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportContractInfoExcel(@Valid RelationsContractInfoPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<RelationsContractInfoDO> list = contractInfoService.getContractInfoPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "员工合同信息.xls", "数据", RelationsContractInfoRespVO.class,
                        BeanUtils.toBean(list, RelationsContractInfoRespVO.class));
    }

}