package cn.iocoder.yudao.module.relations.service.contractinfo;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.relations.controller.admin.contractinfo.vo.RelationsContractInfoPageReqVO;
import cn.iocoder.yudao.module.relations.controller.admin.contractinfo.vo.RelationsContractInfoSaveReqVO;
import cn.iocoder.yudao.module.relations.dal.dataobject.contractinfo.RelationsContractInfoDO;
import cn.iocoder.yudao.module.relations.dal.mysql.contractinfo.RelationsContractInfoMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.relations.enums.ErrorCodeConstants.RELATIONS_CONTRACT_INFO_NOT_EXISTS;

/**
 * 员工合同信息 Service 实现类
 *
 * @author zhaopq
 */
@Service
@Validated
public class RelationsContractInfoServiceImpl implements RelationsContractInfoService {

    @Resource
    private RelationsContractInfoMapper contractInfoMapper;

    @Override
    public Long createContractInfo(RelationsContractInfoSaveReqVO createReqVO) {
        // 插入
        RelationsContractInfoDO contractInfo = BeanUtils.toBean(createReqVO, RelationsContractInfoDO.class);
        contractInfoMapper.insert(contractInfo);
        // 返回
        return contractInfo.getId();
    }

    @Override
    public void updateContractInfo(RelationsContractInfoSaveReqVO updateReqVO) {
        // 校验存在
        validateContractInfoExists(updateReqVO.getId());
        // 更新
        RelationsContractInfoDO updateObj = BeanUtils.toBean(updateReqVO, RelationsContractInfoDO.class);
        contractInfoMapper.updateById(updateObj);
    }

    @Override
    public void deleteContractInfo(Long id) {
        // 校验存在
        validateContractInfoExists(id);
        // 删除
        contractInfoMapper.deleteById(id);
    }

    private void validateContractInfoExists(Long id) {
        if (contractInfoMapper.selectById(id) == null) {
            throw exception(RELATIONS_CONTRACT_INFO_NOT_EXISTS);
        }
    }

    @Override
    public RelationsContractInfoDO getContractInfo(Long id) {
        return contractInfoMapper.selectById(id);
    }

    @Override
    public PageResult<RelationsContractInfoDO> getContractInfoPage(RelationsContractInfoPageReqVO pageReqVO) {
        return contractInfoMapper.selectPage(pageReqVO);
    }

}