package cn.iocoder.yudao.module.system.controller.admin.dictTenant;

import cn.iocoder.yudao.framework.common.enums.CommonStatusEnum;
import cn.iocoder.yudao.module.system.controller.admin.dict.vo.data.DictDataSimpleRespVO;
import cn.iocoder.yudao.module.system.controller.admin.dictTenant.vo.data.DictDataTenantPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.dictTenant.vo.data.DictDataTenantRespVO;
import cn.iocoder.yudao.module.system.controller.admin.dictTenant.vo.data.DictDataTenantSaveReqVO;

import cn.iocoder.yudao.module.system.dal.dataobject.dictTenant.DictDataTenantDO;
import cn.iocoder.yudao.module.system.service.dictTenant.DictDataTenantService;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.*;



@Tag(name = "管理后台 - 租户字典数据")
@RestController
@RequestMapping("/tenant/dict-data")
@Validated
public class DictDataTenantController {

    @Resource
    private DictDataTenantService dictDataTenantService;

    @PostMapping("/create")
    @Operation(summary = "新增租户字典数据")
    @PreAuthorize("@ss.hasPermission('tenant:dict-data:create')")
    public CommonResult<Long> createDictData(@Valid @RequestBody DictDataTenantSaveReqVO createReqVO) {
        return success(dictDataTenantService.createDictData(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "修改租户字典数据")
    @PreAuthorize("@ss.hasPermission('tenant:dict-data:update')")
    public CommonResult<Boolean> updateDictData(@Valid @RequestBody DictDataTenantPageReqVO updateReqVO) {
        dictDataTenantService.updateDictData(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除租户字典数据")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('tenant:dict-data:delete')")
    public CommonResult<Boolean> deleteDictData(@RequestParam("id") Long id) {
        dictDataTenantService.deleteDictData(id);
        return success(true);
    }

    @GetMapping(value = {"/list-all-simple", "simple-list"})
    @Operation(summary = "获得全部字典数据列表", description = "一般用于管理后台缓存字典数据在本地")
    // 无需添加权限认证，因为前端全局都需要
    public CommonResult<List<DictDataSimpleRespVO>> getSimpleDictDataList() {
        List<DictDataTenantDO> list = dictDataTenantService.getDictDataList(
                CommonStatusEnum.ENABLE.getStatus(), null);
        return success(BeanUtils.toBean(list, DictDataSimpleRespVO.class));
    }

    @GetMapping("/get")
    @Operation(summary = "获得租户字典数据")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('tenant:dict-data:query')")
    public CommonResult<DictDataTenantRespVO> getDictData(@RequestParam("id") Long id) {
        DictDataTenantDO dictData = dictDataTenantService.getDictData(id);
        return success(BeanUtils.toBean(dictData, DictDataTenantRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得租户字典数据分页")
    @PreAuthorize("@ss.hasPermission('tenant:dict-data:query')")
    public CommonResult<PageResult<DictDataTenantRespVO>> getDictDataPage(@Valid DictDataTenantPageReqVO pageReqVO) {
        PageResult<DictDataTenantDO> pageResult = dictDataTenantService.getDictDataPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, DictDataTenantRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出租户字典数据 Excel")
    @PreAuthorize("@ss.hasPermission('tenant:dict-data:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportDictDataExcel(@Valid DictDataTenantPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<DictDataTenantDO> list = dictDataTenantService.getDictDataPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "租户字典数据.xls", "数据", DictDataTenantRespVO.class,
                        BeanUtils.toBean(list, DictDataTenantRespVO.class));
    }

}