package cn.iocoder.yudao.module.system.service.dictTenant;

import javax.validation.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.dict.vo.data.DictDataPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.dictTenant.vo.data.DictDataTenantPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.dictTenant.vo.data.DictDataTenantSaveReqVO;


import cn.iocoder.yudao.module.system.dal.dataobject.dictTenant.DictDataTenantDO;
import org.springframework.lang.Nullable;

import java.util.Collection;
import java.util.List;

/**
 * 租户字典数据 Service 接口
 *
 * @author 芋道源码
 */
public interface DictDataTenantService {

    /**
     * 创建租户字典数据
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createDictData(@Valid DictDataTenantSaveReqVO createReqVO);

    /**
     * 更新租户字典数据
     *
     * @param updateReqVO 更新信息
     */
    void updateDictData(@Valid DictDataTenantPageReqVO updateReqVO);

    /**
     * 删除租户字典数据
     *
     * @param id 编号
     */
    void deleteDictData(Long id);

    /**
     * 获得字典数据列表
     *
     * @param status   状态
     * @param dictType 字典类型
     * @return 字典数据全列表
     */
    List<DictDataTenantDO> getDictDataList(@Nullable Integer status, @Nullable String dictType);

    /**
     * 获得租户字典数据
     *
     * @param id 编号
     * @return 租户字典数据
     */
    DictDataTenantDO getDictData(Long id);

    /**
     * 获得租户字典数据分页
     *
     * @param pageReqVO 分页查询
     * @return 租户字典数据分页
     */
    PageResult<DictDataTenantDO> getDictDataPage(DictDataTenantPageReqVO pageReqVO);



    /**
     * 获得指定字典类型的数据数量
     *
     * @param dictType 字典类型
     * @return 数据数量
     */
    long getDictDataCountByDictType(String dictType);

    /**
     * 校验字典数据们是否有效。如下情况，视为无效：
     * 1. 字典数据不存在
     * 2. 字典数据被禁用
     *
     * @param dictType 字典类型
     * @param values   字典数据值的数组
     */
    void validateDictDataList(String dictType, Collection<String> values);

    /**
     * 获得指定的字典数据
     *
     * @param dictType 字典类型
     * @param value    字典数据值
     * @return 字典数据
     */
    DictDataTenantDO getDictData(String dictType, String value);

    /**
     * 解析获得指定的字典数据，从缓存中
     *
     * @param dictType 字典类型
     * @param label    字典数据标签
     * @return 字典数据
     */
    DictDataTenantDO parseDictData(String dictType, String label);

    /**
     * 获得指定数据类型的字典数据列表
     *
     * @param dictType 字典类型
     * @return 字典数据列表
     */
    List<DictDataTenantDO> getDictDataListByDictType(String dictType);

}