package cn.iocoder.yudao.module.system.service.user;


import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.user.vo.tenant.UserTenantRelatePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.user.vo.tenant.UserTenantRelateSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.UserTenantRelateDO;

import javax.validation.Valid;

/**
 * 用户租户关系 Service 接口
 *
 * @author 芋道源码
 */
public interface UserTenantRelateService {

    /**
     * 创建用户租户关系
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createUserTenantRelate(@Valid UserTenantRelateSaveReqVO createReqVO);

    /**
     * 更新用户租户关系
     *
     * @param updateReqVO 更新信息
     */
    void updateUserTenantRelate(@Valid UserTenantRelateSaveReqVO updateReqVO);

    /**
     * 删除用户租户关系
     *
     * @param id 编号
     */
    void deleteUserTenantRelate(Long id);

    /**
     * 获得用户租户关系
     *
     * @param id 编号
     * @return 用户租户关系
     */
    UserTenantRelateDO getUserTenantRelate(Long id);

    /**
     * 获得用户租户关系分页
     *
     * @param pageReqVO 分页查询
     * @return 用户租户关系分页
     */
    PageResult<UserTenantRelateDO> getUserTenantRelatePage(UserTenantRelatePageReqVO pageReqVO);

    /**
     * 将该用户所有关系设为失效
     *
     * @param userId
     * @return
     */
    int deactivateAllForUser(Long userId);

    /**
     * 校验该用户是否在该租户中
     *
     * @param userId
     * @param tenantId
     * @return
     */
    boolean checkUserHasTenant(Long userId, Long tenantId);

    /**
     * 将这个用户这个租户关系设为生效
     *
     * @param queryReqVO
     * @return
     */
    int activateForUser(@Valid UserTenantRelateSaveReqVO queryReqVO);

}